<?php
namespace App\Http\Controllers;

use App\Models\Banner;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;

class BannerController extends Controller {
    /**
     * Get all banners.
     */
    public function getBanners() {
        return response()->json(Banner::all());
    }

    /**
     * Get a single banner by ID.
     */
    public function getBannerById($id) {
        $banner = Banner::find($id);
        if (!$banner) {
            return response()->json(['message' => 'Banner not found'], 404);
        }
        return response()->json($banner);
    }

    /**
     * Create a new banner.
     */
    public function createBanner(Request $request) {
        $request->validate([
            'banner_duration' => 'required|integer',
            'expire_date' => 'required|date',
            'banner_image' => 'required|image|mimes:jpeg,png,jpg,gif|max:2048',
        ]);

        // Upload image
        if ($request->hasFile('banner_image')) {
            $imagePath = $request->file('banner_image')->move(public_path('uploads/banner'), $request->file('banner_image')->getClientOriginalName());
            $imagePath = 'uploads/banner/' . $request->file('banner_image')->getClientOriginalName();

        } else {
            return response()->json(['message' => 'Image upload failed'], 500);
        }

        $banner = Banner::create([
            'banner_duration' => $request->banner_duration,
            'expire_date' => $request->expire_date,
            'banner_image' => $imagePath,
        ]);

        return response()->json(['message' => 'Banner created successfully', 'banner' => $banner], 201);
    }

    /**
     * Update a banner.
     */
    public function updateBanner(Request $request, $id) {
        $banner = Banner::find($id);
        if (!$banner) {
            return response()->json(['message' => 'Banner not found'], 404);
        }

        $request->validate([
            'banner_duration' => 'integer',
            'expire_date' => 'date',
            'banner_image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
        ]);

        // Handle image update
        if ($request->hasFile('banner_image')) {
            // Delete old image
            Storage::disk('public')->delete($banner->banner_image);
            // Upload new image
            $imagePath = $request->file('banner_image')->store('banners', 'public');
        } else {
            $imagePath = $banner->banner_image;
        }

        $banner->update([
            'banner_duration' => $request->banner_duration ?? $banner->banner_duration,
            'expire_date' => $request->expire_date ?? $banner->expire_date,
            'banner_image' => $imagePath,
        ]);

        return response()->json(['message' => 'Banner updated successfully', 'banner' => $banner]);
    }

    /**
     * Delete a banner.
     */
    public function deleteBanner($id) {
        $banner = Banner::find($id);
        if (!$banner) {
            return response()->json(['message' => 'Banner not found'], 404);
        }

        // Delete image
        Storage::disk('public')->delete($banner->banner_image);

        $banner->delete();

        return response()->json(['message' => 'Banner deleted successfully']);
    }
}
