<?php

namespace App\Http\Controllers;

use App\Models\User;
use App\Models\UserGameStats;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Carbon\Carbon;
use Illuminate\Support\Str;


class UserController extends Controller
{

    public function login(Request $request)
{
        $request->validate([
            'email' => 'required|email',
            'password' => 'nullable|string',
            'fullname' => 'nullable|string|max:255', // Needed for new Google users
            'is_google_auth' => 'required|boolean',  // Required to differentiate login type
        ]);

        $user = User::where('email', $request->email)->first();

        if ($user) {
            // 🔹 If user exists
            if (!$request->is_google_auth) {
                // 🔹 Manual login → Validate password
                if (!Hash::check($request->password, $user->password)) {
                    return response()->json([
                        'status' => 'error',
                        'message' => 'Invalid credentials.',
                    ], 401);
                }
            }

            // 🔹 Generate Token and log in
            $token = $user->createToken('auth_token')->plainTextToken;
            $gameStats = UserGameStats::where('user_id', $user->id)->get();

            return response()->json([
                'status' => 'success',
                'message' => 'Login successful',
                'token' => $token,
                'user' => $user,
                'game_stats' => $gameStats,
            ], 200);
        }

        // 🔹 If user doesn't exist & it's Google login → Auto-register
        if ($request->is_google_auth) {
            $user = User::create([
                'fullname' => $request->fullname ?? "Google User",
                'email' => $request->email,
                'password' => Hash::make(Str::random(20)), // Random password (not used)
                'overall_level' => 1,
                'coins' => 0,
            ]);

            $token = $user->createToken('auth_token')->plainTextToken;

            return response()->json([
                'status' => 'success',
                'message' => 'User registered and logged in successfully',
                'token' => $token,
                'user' => $user,
                'game_stats' => [],
            ], 201);
        }

        // 🔹 If manual login & user doesn't exist → Show error
        return response()->json([
            'status' => 'error',
            'message' => 'User not found. Please sign up.',
        ], 404);
    }


    /**
     * Logout User (Invalidate Token)
     */
    public function logout(Request $request)
    {
        $user = $request->user();
        
        if ($user) {
            $user->tokens()->delete(); // Revoke all user tokens
        }

        return response()->json(['status' => 'success', 'message' => 'Logged out successfully']);
    }


    /**
     * Get all users.
     */
    public function getUsers()
    {
        $users = User::all();
        return response()->json($users);
    }

    /**
     * Get a single user by ID.
     */
    public function getUserById($id)
    {
        $user = User::find($id);
        if (!$user) {
            return response()->json(['message' => 'User not found'], 404);
        }

        return response()->json($user);
    }

    /**
     * Create a new user.
     */
    
    public function register(Request $request)
    {
        $request->validate([
            'fullname' => 'required|string|max:255',
            'email' => 'required|email',
            'password' => 'required|string|min:6|confirmed',
            'game_stats' => 'nullable|array', // Expecting an array of game stats
        ]);

        $isGoogleSignup = $request->password === 'google_auth_Hv5BWNt';

        // 🔹 If user exists and it's a Google signup, log them in
        $user = User::where('email', $request->email)->first();
        if ($user) {
            if ($isGoogleSignup) {
                // 🔹 Google Signup → Auto Login
                $token = $user->createToken('auth_token')->plainTextToken;

                return response()->json([
                    'status' => 'success',
                    'message' => 'User logged in successfully',
                    'token' => $token,
                    'user' => $user,
                    'game_stats' => $user->gameStats,
                ], 200);
            } else {
                // 🔹 Manual Signup → Show "Email already exists" error
                return response()->json([
                    'status' => 'error',
                    'message' => 'Email already exists. Please log in instead.',
                ], 422);
            }
        }
    
        // Create User
        $user = User::create([
            'fullname' => $request->fullname,
            'email' => $request->email,
            'password' => $isGoogleSignup ? Hash::make(Str::random(20)) : Hash::make($request->password),
            'overall_level' => 1,
            'coins' => 0,
        ]);
    
        // Store Game Stats (if provided)
        $totalGamesPlayed = 0;
        $weightedLevelSum = 0;
        // return response()->json($request->all(), 200);

        if ($request->filled('game_stats')) {
            foreach ($request->game_stats as $game) {
                $userGameStat = UserGameStats::updateOrCreate(
                    ['user_id' => $user->id, 'game_name' => $game['game_name']],
                    [
                        'games_played' => $game['games_played'],
                        'games_won' => $game['games_won'],
                        'best_try' => $game['best_try'],
                        'current_streak' => $game['current_streak'],
                        'max_streak' => $game['max_streak'],
                        'last_played_at' => Carbon::parse($game['last_played_at'])->format('Y-m-d H:i:s'),
                        'level' => $game['level'],
                    ]
                );
    
                // Calculate Overall Level using Weighted Average
                $totalGamesPlayed += $game['games_played'];
                $weightedLevelSum += $game['level'] * $game['games_played'];
            }
        }
    
        // Calculate and Update User's Overall Level
        if ($totalGamesPlayed > 0) {
            $user->overall_level = round($weightedLevelSum / $totalGamesPlayed);
            $user->save();
        }
    
        // Generate Token
        $token = $user->createToken('auth_token')->plainTextToken;
    
        return response()->json([
            'status' => 'success',
            'message' => 'User registered successfully',
            'token' => $token,
            'user' => $user,
            'game_stats' => $user->gameStats, // Send back saved game stats
        ], 201);
    }
    


    /**
     * Update an existing user.
     */
    public function updateUser(Request $request, $id)
    {
        $user = User::find($id);
        if (!$user) {
            return response()->json(['message' => 'User not found'], 404);
        }

        $request->validate([
            'fullname' => 'string|max:255',
            'email' => 'email|unique:users,email,' . $id,
            'password' => 'nullable|string|min:6',
        ]);

        $user->update([
            'fullname' => $request->fullname ?? $user->fullname,
            'email' => $request->email ?? $user->email,
            'password' => $request->password ? Hash::make($request->password) : $user->password,
        ]);

        return response()->json(['message' => 'User updated successfully', 'user' => $user]);
    }

    /**
     * Delete a user.
     */
    public function deleteUser($id)
    {
        $user = User::find($id);
        if (!$user) {
            return response()->json(['message' => 'User not found'], 404);
        }

        $user->delete();

        return response()->json(['message' => 'User deleted successfully']);
    }
}
